#include<iostream>
#include<fstream>

#include<LEDA/graph/graph.h>
#include<LEDA/graph/ugraph.h>
#include<LEDA/graphics/graphwin.h>
#include<LEDA/graph/graph_alg.h>
#include<LEDA/graphics/window.h>
#include<LEDA/graph/node_partition.h>

#include<LEDA/graph/node_array.h>
#include<LEDA/core/set.h>
#include<LEDA/core/array.h>
#include<LEDA/core/string.h>
#include<LEDA/core/stack.h>
#include<LEDA/core/list.h>

#include<LEDA/numbers/integer.h>
#include<LEDA/numbers/integer_matrix.h>
#include<LEDA/core/random_source.h>


using namespace leda;
using std::cout;
using std::cin;
using std::endl;


class Minset{

private:

  GRAPH<int,int> H;
  node_array< list<node> > GenList;

  node v,w;
  edge e;

  set<node> minset;

  int minsize;
  node mins,mint;


private:

  set<node> st_sep_set(GRAPH<string,int>& U,node is,node it);
    
public:

  set<node> set_minset(UGRAPH<string,int> & U);

};


class TComp{

public:
  
  TComp(UGRAPH<string,int>& inG);
  void heuristic();
  UGRAPH< set<node> ,int>& monmon(){
    return TX;
  }

private:
  
  UGRAPH< set<node> ,int> TX;
  UGRAPH< string,int> Ginit;
  node_array< int > flag;

  node_array<double> xpos;
  node_array<double> ypos;


  GraphWin gw,gw2,gw3;
  int gw_font_size;

  void post_process();

  UGRAPH<string,int> searchGi(node& nodei);
  void printTX();
  void print_Xarray(array< set<node> > Xarray,int max);
  set<node> srcset(set<node>& inS);
  set<node> tagset(UGRAPH<string,int>& UG,set<node>& adjS);
  stack<edge> edge_stack;


  void view();
  void view_Gi(UGRAPH<string,int>& UG);

};

bool Is_Complete(UGRAPH<string,int>& UG);

void print_Xi_cout(GraphWin& gw, const point&)
{ 
  string s;
  node v  = gw.get_edit_node();
  int col = (gw.get_color(v) + 1) % 16;
  gw.set_color(v,color(col));
 
  s = gw.get_label(v);

 }



TComp::TComp(UGRAPH<string,int>& inG){

  node tmpv,tmpw;

    
  Ginit = inG;  

  gw_font_size = 16;
  
  tmpv = TX.new_node();
  
  forall_nodes(tmpw,Ginit){
    
    TX[tmpv].insert(tmpw);
    
  }


  
}



void TComp::heuristic(){
  
  Minset Ms;

  UGRAPH<string,int> Gi;
  set<node> minset;
  set<node> minset2;
  
  int max;

  node_array<int> components;
  
  array< set<node> > Xarray;
  array< set<node> > Yarray;
  array< set<node> > sXarray;
  array< set<node> > sYarray;
  
  node tmpv,tmpw;

  node nodei;
  node root;

  flag.init(TX,1000,0);
  xpos.init(TX,1000,200);
  ypos.init(TX,1000,200);

  
  random_source RS(0,400);


  do{

    Gi = searchGi(nodei);



    if(Gi.number_of_nodes() == 0) {
      
      int width = 0;
      
      forall_nodes(tmpv,TX){
	
	if(TX[tmpv].size() > width){
	  width = TX[tmpv].size();
	}
      }
      
      width--;
      
      post_process();
      
      view();
      
      
      return;
    }
    
    minset = Ms.set_minset(Gi);
    
    
    minset2 = srcset(minset);
    
    forall_nodes(tmpv,Gi){
      if(minset.member(tmpv)){
	Gi.hide_node(tmpv);
      }
    }
    
    components.init(Gi);
    COMPONENTS(Gi,components);
    
    
    max = 0;
    forall_nodes(tmpv,Gi)
      if(max < components[tmpv]) max = components[tmpv];
    
    Xarray.resize(0);
    Yarray.resize(0);
    sXarray.resize(0);
    sYarray.resize(0);
    
    Xarray.resize(max+2);
    Yarray.resize(max+2);
    sXarray.resize(max+2);
    sYarray.resize(max+2);
    
    
    Yarray[0] = minset;
    for(int i=1;i < max + 2;i++){
      
      forall_nodes(tmpv,Gi){      
	if(i == components[tmpv] + 1) {
	  Yarray[i].insert(tmpv);
	} 
      }
    }
    
    
    for(int i=0;i < max + 2;i++){
      Xarray[i] = Yarray[i] + minset;
    }
    
    
    
    
    for(int i=0;i < max + 2;i++){
      sYarray[i] = srcset(Yarray[i]);
    }
    
    
    for(int i=0;i < max + 2;i++){
      sXarray[i] = srcset(Xarray[i]);
    }

    root = TX.new_node();
    xpos[root] = RS(0,400);
    ypos[root] = RS(100,500);
    
    flag[nodei] = 1;
    
    
    TX[root] = sXarray[0];
    
    for(int i=1;i < max+2;i++){
      tmpv = TX.new_node();
      TX[tmpv] = sXarray[i];
      TX.new_edge(root,tmpv);
      xpos[tmpv] = RS(0,400);
      ypos[tmpv] = RS(100,500); 
    }
    
    forall_adj_nodes(tmpv,nodei){
      bool find = false;
      
      forall_adj_nodes(tmpw,root){
	
	if( (  (TX[tmpw] - TX[root]) & TX[tmpv] ).empty() ==false  )
	  {
	    TX.new_edge(tmpw,tmpv);
	    find = true;
	  }
	
      }
      
      if(find == false){
	TX.new_edge(root,tmpv);
      }
      
    }    
    
    TX.del_node(nodei);

  }while(true);
  
  
  
}

UGRAPH<string,int> TComp::searchGi(node& nodei){

  UGRAPH<string,int> Gi;
  set<node> InduceSet;
 

  node tmpv,tmpw,tmpx,tmpv2,tmpw2;
  edge tmpe;

  UGRAPH<string,int> maxGi;
  node maxv;

  UGRAPH<string,int> emptyG;
  
  bool find2 = false;

  if(TX.number_of_nodes() == 1){
    Gi = Ginit;
    nodei = TX.first_node();

    return Gi;
  }
 
  else{
    
    forall_nodes(tmpv,TX)
      {	
	if(flag[tmpv] != 2){

	  InduceSet = TX[tmpv];
	  set<node> AdjSet;
	  
	  forall_adj_nodes(tmpw,tmpv){
	    
	    AdjSet += InduceSet & TX[tmpw];
	  }
	  
	  forall_nodes(tmpw,Ginit){
	    if( !TX[tmpv].member(tmpw) )
	      {
		Ginit.hide_node(tmpw);
	      }
	  }
	  
	  
	  Gi = Ginit;
	  nodei = tmpv;

	  Ginit.restore_all_nodes();
	  Ginit.restore_all_edges();
	  
	  set<node> tagadjset = tagset(Gi,AdjSet);
	 


	  forall(tmpv2,tagadjset){
	    forall(tmpw2,tagadjset){
	      if((tmpv2 != tmpw2) && ( (Gi.adj_nodes(tmpv2) ).rank(tmpw2) == 0))
		{
		  tmpe = Gi.new_edge(tmpv2,tmpw2);
		  edge_stack.push(tmpe);
		  
		}
	    }
	  }



	  
	  if(Is_Complete(Gi)){
	    flag[nodei] = 2;
	  }
	  
	  else if(Is_Complete(Gi) == false && ( Gi.number_of_nodes() > maxGi.number_of_nodes())  ){  
	    
	    maxGi = Gi;
	    maxv = nodei;
	    find2 = true;

	  }
	  
	  

	}

      }

    

    if(find2){
      nodei = maxv;
      return maxGi;
    }
    
  }
  return emptyG;

}



void TComp::post_process(){

  node tmpv,tmpw;
  set<node> AdjSet;

  array< set<node> > Set(2);
  array<node> AdjN(2); 
  int i,tmpi;
  stack<node> estack;


  forall_nodes(tmpv,TX){

    i=0;
    Set[0].clear();
    Set[1].clear();


    if(TX.degree(tmpv) == 2){

      forall_adj_nodes(tmpw,tmpv){
	Set[i] = TX[tmpw];
	AdjN[i] = tmpw;

	i++;
      }
   
      if(TX[tmpv] <= (Set[0] & Set[1] ) )
	{
	  TX.new_edge(AdjN[0],AdjN[1]);
	  estack.push(tmpv);
	}
    }
    
  }
  
  do{
    
    if(estack.empty() == true) break;

    TX.del_node(estack.pop() );

  }
  while(true);

}


void TComp::view(){

  node tmpv,tmpw;
  string tmps;
  int tmpi =0;

  gw.set_graph(TX);
  gw.set_node_label_type(user_label);
  gw.set_node_shape(rectangle_node);
  gw.set_node_label_font(roman_font,gw_font_size);



  forall_nodes(tmpv,TX){
    
    tmpi = 0;
    tmps ="";


    forall(tmpw,TX[tmpv]){
      if(tmpi > 0) tmps += ", ";
      tmps += Ginit[tmpw];
      tmpi++;
    }
    
    if(flag[tmpv] == 1){
      gw.set_color(tmpv,blue);
    }


    if(flag[tmpv] == 2){
      gw.set_color(tmpv,green);
    }


    if(flag[tmpv] == 10){
      gw.set_color(tmpv,red);
    }

    gw.set_label(tmpv,tmps);    

    gw.set_width(tmpv,tmps.length()*(gw_font_size / 1.7));



  }
  gw.set_edge_direction(undirected_edge);
  gw.set_position(xpos,ypos);

  gw.set_action(A_LEFT | A_NODE | A_CTRL, print_Xi_cout);
 
  gw.display();
  gw.edit();  

}


void TComp::printTX(){

  node tmpv,tmpw;
  int tmpi;

  tmpi = 0;


  forall_nodes(tmpv,TX){
    
    forall(tmpw,TX[tmpv]){
    }
    cout << endl;
    tmpi++;
  }
  
  
}

set<node> TComp::srcset(set<node>& inS){

  set<node> srcset;
  node tmpv,tmpw;

  forall(tmpv,inS){
    
    forall_nodes(tmpw,Ginit){
      if(Ginit[tmpv] == Ginit[tmpw])
	{
	  srcset.insert(tmpw);
	}
    }
  }
  return srcset;
}


set<node> TComp::tagset(UGRAPH<string,int>& UG,set<node>& adjS){

  set<node> tagset;
  node tmpv,tmpw;

  forall(tmpv,adjS){
    
    forall_nodes(tmpw,UG){
      if(Ginit[tmpv] == UG[tmpw])
	{
	  tagset.insert(tmpw);
	}
    }
  }
  return tagset;
}


void TComp::view_Gi(UGRAPH<string,int>& UG){

  
  gw2.set_graph(UG);
  gw2.set_node_label_type(data_label);
  gw2.set_node_label_font(bold_font,gw_font_size);
  gw2.set_edge_direction(undirected_edge);
  gw2.set_node_color(orange);
  gw2.set_node_label_color(blue);

  gw2.set_bg_color(grey2);
  
  
  gw2.display();
  gw2.edit();

}

set<node> Minset::st_sep_set(GRAPH<string,int>& U,node is,node it){

    

  node s = GenList[is].head();
  node t = GenList[it].tail();
  
  edge_array<int> f(H,0);
  edge tmpe;
  node tmpv;

  set<node> stS;
  
  node_array<bool> Sep;
    
    
  Sep.init(H,false);
  MAX_FLOW(H,s,t,H.edge_data(),f);
  
  forall_edges(tmpe,H){

    
    if(H[tmpe] == f[tmpe]){
      

      Sep[H.source(tmpe)] = true;
      Sep[H.target(tmpe)] = true;
      
      
    }
  }

  
  forall_nodes(tmpv,U){
    
    if(Sep[ GenList[tmpv].head() ]   && Sep[ GenList[tmpv].tail() ] )
      {
	stS.insert(tmpv);
      }
    
    
    }
  
  return stS;
    
}


set<node> Minset::set_minset(UGRAPH<string,int> & U){
    
  GenList.init( U );
  
  set<node> stS;
  
  set<node> S;
  
  node s,t;
  node tmpv;
  node tmpvs,tmpvt;
  
    
  int i,j;
  
  forall_nodes(v,U){
    
    node n1,n2;
    
    n1 = H.new_node();
    n2 = H.new_node();
    
    e = H.new_edge(n1,n2);
    H[e] = 1;
	
    GenList[v].push(n1);
    GenList[v].push(n2);
    
  }
  forall_nodes(v,U){
    
    edge tmpe;
    node adjv;
    
    forall_adj_nodes(adjv,v){
      
      tmpe = H.new_edge(GenList[v].head() , GenList[adjv].tail() );
      H[tmpe] = 100000;
      
    }    
  }
  
  i=0;
  j=0;
  
  minsize = 100000;
  
  forall_nodes(tmpvs,U){       
	
    forall_nodes(tmpvt,U){
      
      if(tmpvs != tmpvt && ( U.adj_nodes(tmpvs) ).rank(tmpvt) == 0  )
	{
	  s = tmpvs;
	  t = tmpvt;
	  stS = st_sep_set(U,s,t);
	  
	  if(stS.size() < minsize){
	    mins = s;
	    mint = t;
	    minsize = stS.size();
	    minset = stS;		
	  }
	}
      
      j++;
    }
    i++;
    j = 0;
  }
  return minset;
}


bool Is_Complete(UGRAPH<string,int>& UG){
  
  node tmpv;

  forall_nodes(tmpv,UG){
    if(UG.degree(tmpv) != UG.number_of_nodes() - 1 )
      return false;
  }

  return true;

}

/***********************************************************************/

class sepa2{
public:
  int size;
  int **sp;
  integer_matrix **Cr;
  integer_matrix d;
  sepa2(){
    size = 2;

    sp = new int*[2];
    for(int i = 0; i < 2; i++)
      sp[i] = new int[2];

    sp[0][0] = 0; sp[0][1] = 0;
    sp[1][0] = 0; sp[1][1] = 1;

    integer_matrix M1(2,1);

    M1(0,0) = -1; M1(1,0) = 1;

    integer_matrix M2(1,1);

    M2(0,0) = -1;

    integer_matrix M3(1,2);

    M3(0,0) = -1; M3(0,1) = 1;
    
    Cr = new integer_matrix*[size];
    for(int i = 0; i < size; i++)
      Cr[i] = new integer_matrix[size];

    Cr[0][0] = M1;  Cr[0][1] = M2;
    Cr[1][0] = M2;  Cr[1][1] = M3;

    integer_matrix dt(3,2);

    for(int i = 0; i < 3; i++)
      for(int j = 0; j < 2; j++)
	dt(i,j) = 0;

    dt(0,1) = 1;

    dt(1,0) = 1; dt(1,1) = -2;

    dt(2,0) = -1; dt(2,1) = 1;

    d = dt;

  }
};


void set_G(UGRAPH<set<node>,int> &, GRAPH<string,int> &, node_array< set<edge> > &);
integer_matrix TP(node , UGRAPH<set<node>,int> &, GRAPH<string,int> &, node_array<set<edge> >, node_partition &, graph);
void partition_match(GRAPH<string, int> &, node_partition &, node_partition &);
void partition_union(node_partition &, array<node> &, int *, int);
void graph_identify(node, UGRAPH<set<node> ,int> &, GRAPH<string,int> &, node_array<set<edge> > ,node_partition &, graph &);
void link_nodes(UGRAPH<set<node>, int> &, node, set<node>&);
void identify(graph &, node, node);
integer_matrix sf(array<integer_matrix> , int*, array<integer_matrix> , int*, integer_matrix**, int, integer_matrix, graph);
integer_matrix x_1(int a);
integer_matrix multi_poly(integer_matrix, integer_matrix);
integer_matrix add_poly(integer_matrix, integer_matrix);
integer_matrix div_poly(integer_matrix T1, integer_matrix T2);
integer_matrix tutte_pl(graph);
void subset(graph &, int, int, int, int, array<edge> &, array<edge> &, int, integer_matrix &);
void tutte(graph &,int , int , int ,array<edge> &,integer_matrix &);
void tutte_coe(int , int , integer_matrix &);
integer kaijo(int);
integer pow_int(int, int);
integer combi(int, int);

int main(){
  
  UGRAPH<string,int> G;
  
  G.read("graph1.gw");
 
  TComp TX(G);
  
  TX.heuristic();
  
  UGRAPH< set<node>, int > GT;
  
  GT = TX.monmon();
  
  node_array<set<edge> > se(GT);
  
  G.make_directed();

  node_partition P(G);
  
  set_G(GT, G, se);
  
  integer_matrix T;

  T = TP(GT.first_node(), GT, G, se, P, G);
 
  cout << "T = " << endl;

  for(int i = 0; i < T.dim1(); i++){
    for(int j = 0; j < T.dim2(); j++)
      cout << T(i,j) << " ";
    cout << endl;
  }
 

  integer_matrix T1;

  T1 = tutte_pl(G);

  cout << "T1 = " << endl;

  for(int i = 0; i < T1.dim1(); i++){
    for(int j = 0; j < T1.dim2(); j++)
      cout << T1(i,j) << " ";
    cout << endl;
  }

  return 0;
}


void set_G(UGRAPH<set<node>,int> &GT,GRAPH<string,int> &G,node_array< set<edge> > &se){
  
  node n;
  
  list<edge> L;
  
  set<node> Sal; 
  
  forall_nodes(n, G){
    Sal.insert(n);
  }
  
  forall_nodes(n, GT){
    
    se[n].clear();
    
    set<node> St;
    set<node> S;
    
    node n1;
    forall(n1, GT[n]){
      
      node n2;
      forall_nodes(n2,G)
	if(index(n1) == index(n2))
	  break;
      
      S.insert(n2);
    }
    
    St = Sal-S;
    
    GT[n] = S;
    
    forall(n1,St) 
      G.hide_node(n1);
    
    edge e;
    forall_edges(e, G){
      se[n].insert(e);
      L.push(e);
    }
    
    G.restore_all_nodes();
    G.restore_all_edges();
    G.hide_edges(L);
  }
  
  G.restore_all_nodes();
  G.restore_all_edges();
  
  forall_nodes(n, GT){
    
    cout << "node = ";
    node n1;
    forall(n1, GT[n])
      cout << index(n1) << ' ';
    cout << endl;
    
    cout << "edge = ";
    edge e;
    forall(e,se[n])
      cout << index(e) << ' ';
    cout << endl;
  }
  
}


integer_matrix TP(node Ti, UGRAPH<set<node>,int> &GT, GRAPH<string,int> &G,node_array<set<edge> > se, node_partition &P,graph Gx){

  if(GT.degree(Ti)>=1){
    edge e;
    e = GT.first_adj_edge(Ti);
    
    node n2;
    n2 = GT.opposite(Ti,e);
    
    set<node> S;
    
    S = GT[Ti]&GT[n2];
  
    set<node> S2;

    while(S.size()){

      node n3 = S.choose();
      S.del(n3);
      S2.insert(n3);
      set<node> S3 = S;

      while(S3.size()){

	node n4 = S3.choose();
	S3.del(n4);

	if(P.same_block(n3,n4))
	  S.del(n4);
      }
    }
    
    S = S2;
 
    GT.hide_edge(e);
    
    int p = S.size();
    
    if(p == 2){
      
      sepa2 ob;
      int x = ob.size;
      array<integer_matrix> T1(x);
      array<integer_matrix> T2(x);
      int c1[x];
      int c2[x];

      array<node> pn(p);

      int i = 0;
      node n3;
      forall(n3, S){
	pn[i] = n3;
	i++;
      }
      
      for(i = 0; i < x; i++){

	node_partition P1(G);
	partition_match(G,P,P1);

	partition_union(P1, pn, ob.sp[i],p);

	graph Gy;

	graph_identify(Ti,GT,G,se,P1,Gy);
 
	node_array<int> comp_num(Gy);
	
	c1[i] = COMPONENTS(Gy,comp_num);
	
	T1[i] = TP(Ti, GT, G, se, P1, Gy);

      }

      for(i = 0; i < x; i++){

	node_partition P1(G);
	partition_match(G,P,P1);

	partition_union(P1, pn, ob.sp[i],p);

	graph Gy;

	graph_identify(n2,GT,G,se,P1,Gy);
  
	node_array<int> comp_num(Gy);

	c2[i] = COMPONENTS(Gy,comp_num);

	T2[i] = TP(n2, GT, G, se, P1, Gy);

      }

      GT.restore_edge(e);

      return(sf(T1, c1, T2, c2, ob.Cr, x, ob.d, Gx));

    }
    else {

      integer_matrix T1;
      integer_matrix T2;

	graph Gy;

	graph_identify(Ti,GT,G,se,P,Gy);

	T1 = TP(Ti, GT, G, se, P, Gy);

	graph_identify(n2,GT,G,se,P,Gy);

	T2 = TP(n2, GT, G, se, P, Gy);

	GT.restore_edge(e);

	return (multi_poly(T1,T2));
    }
  }
  else    {
    return(tutte_pl(Gx));
  }
}




void partition_match(GRAPH<string, int> &G, node_partition &P, node_partition &P1){


  node n;
  int node_num = 1; 
  forall_nodes(n, G){
    if(n != G.last_node()){
      node n1 = G.succ_node(n);
      for(int i = node_num + 1; i <= G.number_of_nodes(); i++){
	if(P.same_block(n,n1))
	  P1.union_blocks(n,n1);
	n1 = G.succ_node(n1);
      }
    }
    node_num++;
  }


}



void partition_union(node_partition &P1, array<node> &pn, int *sepa, int p_size){


  for(int i = 0; i < p_size-1; i++)
    for(int j = i+1; j < p_size; j++)
      if( sepa[i] == sepa[j] && !(P1.same_block(pn[i],pn[j])) )
	P1.union_blocks(pn[i],pn[j]);
   
}


void graph_identify(node n, UGRAPH<set<node> ,int> &Gn, GRAPH<string,int> &G, node_array<set<edge> > se,node_partition &P1, graph &Gy){

 
  set<node> S;

  link_nodes(Gn,n,S);

  set<node> Sal;
  list<edge> Lal;

  node n2;
  forall(n2, S)
    Sal += Gn[n2];

  forall(n2, S){
    edge e;
    forall(e, se[n2])
      Lal.append(e);
  }

  forall_nodes(n2,G)
    G.hide_node(n2);

  forall(n2,Sal){
    G.restore_node(n2);
  }

  G.restore_edges(Lal);

  GRAPH<string,int> Gt = G;
  
  G.restore_all_nodes();
  G.restore_all_edges();
  
  while(Sal.size()){
    
    n2 = Sal.choose();
    Sal.del(n2);
    set<node> St = Sal;
    
    while(St.size()){

      node n3 = St.choose();
      St.del(n3);

      if(P1.same_block(n2,n3)){

	node n4,n5,n6;
	
	forall_nodes(n4,Gt){
	  if(Gt[n4] == G[n2])
	    n5 = n4;
	  if(Gt[n4] == G[n3])
	    n6 = n4;
	}
	identify(Gt,n5,n6);
	
	Sal.del(n3);
      }
    }
  }

  Gy = Gt;

}


void link_nodes(UGRAPH<set<node>, int> &Gn, node n, set<node> &S){
  
  
  S.insert(n);

  if(Gn.degree(n) >= 1){

    list<node> Ln;
    list<edge> Le;
    
    Ln = Gn.adj_nodes(n);
    Le = Gn.adj_edges(n);

    Gn.hide_edges(Le);
    
    node u;
    forall(u,Ln)
      link_nodes(Gn,u,S);
    
    Gn.restore_edges(Le);
  }
}

void identify(graph &Gt, node n1, node n2){
  

  edge e;
  forall_inout_edges(e, n2){
    node n3 = Gt.opposite(n2, e);
    Gt.move_edge(e, n1, n3);
  }

  Gt.del_node(n2);

}


integer_matrix sf(array<integer_matrix> T1, int* c1, array<integer_matrix> T2, int* c2, integer_matrix ** Cr, int x, integer_matrix d, graph Gx){


  node_array<int> comp_num(Gx);
  int N = COMPONENTS(Gx,comp_num);
  
  integer_matrix dx;

  dx = multi_poly(d, x_1(N));

  array<integer_matrix> kr(x);
  array<integer_matrix> hr(x);

  for(int i = 0; i < x; i++){
    
    kr[i] = multi_poly(x_1(c1[i]),T1[i]);
    
    hr[i] = multi_poly(x_1(c2[i]),T2[i]);
    
  }

  array<integer_matrix> krCr(x);
  
  for(int i = 0; i < x; i++)
    for(int j = 0; j < x; j++)
      krCr[i] = add_poly( krCr[i],multi_poly(kr[j], Cr[j][i]));
  
  
  integer_matrix krCrhr;


  for(int i = 0; i < x; i++)
    krCrhr = add_poly( krCrhr, multi_poly(krCr[i],hr[i]));

  return( div_poly(krCrhr,dx));
  
}

integer_matrix x_1(int a){
  
  integer_matrix x_1(a+1,1);
  
  for(int i = 0; i <= a; i++)
    x_1(i,0) += ((combi(a, i)) * (pow_int(-1,a-i)));
  
  return x_1;
  
}


integer_matrix multi_poly(integer_matrix  T1, integer_matrix  T2){
  
  
  integer_matrix T(T1.dim1() + T2.dim1() - 1, T1.dim2() + T2.dim2() - 1);
  
  for(int i = 0; i < T1.dim1(); i++)
    for(int j = 0; j < T1.dim2(); j++)
      for(int k = 0; k < T2.dim1(); k++)
	for(int l = 0; l < T2.dim2(); l++)
	  if(T1(i,j) != 0 && T2(k,l) != 0)
	    T(i+k,j+l) += T1(i,j)*T2(k,l);  
  
  return T;
  
}

integer_matrix add_poly(integer_matrix T1, integer_matrix T2){
  
  int row;
  int col;
  
  if(T1.dim1() > T2.dim1())
    row = T1.dim1();
  else row = T2.dim1();
  
  if(T1.dim2() > T2.dim2())
    col = T1.dim2();
  else col = T2.dim2();
  
  integer_matrix T(row,col);
  
  for(int i = 0; i < T1.dim1(); i++)
    for(int j = 0; j < T1.dim2(); j++)
      T(i,j) = T1(i,j);
  
  for(int i = 0; i < T2.dim1(); i++)
    for(int j = 0; j < T2.dim2(); j++)
      T(i,j) += T2(i,j);
  
  return T;
  
}



integer_matrix div_poly(integer_matrix T1, integer_matrix T2){

  int x1, x2, y1, y2, sftx, sfty, check, i, j;
  integer tim;
  integer_matrix T;

  for(j = T2.dim2()-1, check = 0; j >= 0; j--){
    for(i = T2.dim1()-1; i >= 0; i--)
      if(T2(i,j)!=0){
        x2 = i;
	y2 = j;
	check = 1;
	break;
      }    
    if(check == 1)
      break;
  }


  while(1){
    
    for(j = T1.dim2()-1, check = 0; j >= 0; j--){
      for(i = T1.dim1()-1; i >= 0; i--)
	if(T1(i,j)!=0){
	  x1 = i;
	  y1 = j;
	  check = 1;
	  break;
	}    
      if(check == 1)
	break;
    }
    if(i == -1 && j == -1){
      
      return T;
    }
    
    sftx = x1 - x2;
    sfty = y1 - y2;
    tim = T1(x1,y1)/T2(x2,y2);
    
    integer_matrix Tt(sftx+1,sfty+1);
    
    Tt(sftx,sfty) = tim;
    
    T = add_poly(T, Tt);
    
    for(i = 0; i < T2.dim1(); i++)
      for(j = 0; j < T2.dim2(); j++)
	T1(i+sftx,j+sfty) -= tim * T2(i,j);
    
  }
}

  
// Tutte多項式の計算

integer_matrix tutte_pl(graph G){

  int n = G.number_of_nodes();
  int m = G.number_of_edges();

  node_array<int> comp_num(G);
  int N = COMPONENTS(G, comp_num);

  int i = 0;

  array<edge> a(m);
  array<edge> s(m);

  edge e;
  forall_edges(e, G){
    a[i] = e;
    i++;
  }
  
  integer_matrix T(n-N+1,m-(n-N)+1);

  for(i = 0; i < T.dim1(); i++)
    for(int j = 0; j < T.dim2(); j++)
      T(i,j) = 0;
  
  subset(G, n, N, 0, 0, a, s, m, T);

  return T;

}

void subset(graph &G, int n, int N, int i, int h, array<edge> &a, array<edge> &s, int m,  integer_matrix &T) {

  if (i > m - 1) {
    tutte(G,n,N,h - 1,s,T);
  }
  else {
    subset(G,n,N,i + 1,h,a,s,m,T);
    s[h] = a[i];   subset(G,n,N,i + 1,h + 1,a,s,m,T);
  }

}

void tutte(graph &G,int n, int N, int l,array<edge> &s,integer_matrix &T) {


  int i;
  for (i = 0; i <= l; i++) {
    G.hide_edge(s[i]);
  }
  node_array<int> comp_num(G);
  int N2 = COMPONENTS(G, comp_num);
  int m2 = G.number_of_edges();
  tutte_coe((n-N)-(n-N2), m2-(n-N2), T);
  G.restore_all_edges();


}

void tutte_coe(int a, int b, integer_matrix &T)
{
  int i, j;

  for(i = 0; i <= a; i++)
    for(j = 0; j <= b; j++)
      T(i,j) += ((combi(a, i)) * (pow_int(-1,a-i)))*
	((combi(b, j)) * (pow_int(-1,b-j)));
  
}

integer pow_int(int a, int b)
{
  int i;
  integer n = 1;

  for(i = 0; i < b; i++)
    n *= a;
  return n;
}

integer combi(int n, int k)
{
  return kaijo(n)/(kaijo(n-k)*kaijo(k));
}

integer kaijo(int a)
{
  int i;
  integer n = 1;
  
  for(i = 2; i <= a; i++)
    n *= i;
  return n;
}
